@@ -468,7 +468,12 @@ class S3FileSystem(AbstractFileSystem):
                     'VersionId': out.get('VersionId')
                 }
             except ClientError as e:
-                raise translate_boto_error(e)
+                ee = translate_boto_error(e)
+                # This could have failed since the thing we are looking for is a prefix.
+                if isinstance(ee, FileNotFoundError):
+                    return super().info(path)
+                else:
+                    raise ee
             except ParamValidationError as e:
                 raise ValueError('Failed to head path %r: %s' % (path, e))
         return super().info(path)
@@ -689,8 +694,8 @@ class S3FileSystem(AbstractFileSystem):
         """
         bucket, key = split_path(path)
         return self.s3.generate_presigned_url(
-            ClientMethod='get_object', Params=dict(Bucket=bucket, Key=key,
-                                                   **kwargs),
+            ClientMethod='get_object',
+            Params=dict(Bucket=bucket, Key=key, **kwargs),
             ExpiresIn=expires)
 
     def merge(self, path, filelist, **kwargs):
@@ -849,10 +854,10 @@ class S3FileSystem(AbstractFileSystem):
             self.dircache.pop(path, None)
             self.dircache.pop(self._parent(path), None)
 
-    def walk(self, path, maxdepth=None):
+    def walk(self, path, maxdepth=None, **kwargs):
         if path in ['', '*', 's3://']:
             raise ValueError('Cannot crawl all of S3')
-        return super().walk(path, maxdepth=maxdepth)
+        return super().walk(path, maxdepth=maxdepth, **kwargs)
 
 
 class S3File(AbstractBufferedFile):
@@ -923,6 +928,13 @@ class S3File(AbstractBufferedFile):
                 self.size = self.details['size']
             elif self.fs.version_aware:
                 self.version_id = self.details.get('VersionId')
+                # In this case we have not managed to get the VersionId out of details and 
+                # we should invalidate the cache and perform a full head_object since it 
+                # has likely been partially populated by ls.
+                if self.version_id is None:
+                    self.fs.invalidate_cache(self.path)
+                    self.details = self.fs.info(self.path)
+                    self.version_id = self.details.get('VersionId')
 
         if 'a' in mode and s3.exists(path):
             loc = s3.info(path)['size']
@@ -962,7 +974,7 @@ class S3File(AbstractBufferedFile):
                     self.fs.s3.upload_part_copy,
                     self.s3_additional_kwargs,
                     Bucket=self.bucket,
-                    Key=self.key, 
+                    Key=self.key,
                     PartNumber=1,
                     UploadId=self.mpu['UploadId'],
                     CopySource=self.path)
